@@ -51,9 +51,9 @@ use std::mem::replace;
 /// A priority queue with efficient change function to change the priority of an
 /// element.
 ///
-/// The priority is of type P, that must implement `std::cmp::Ord`.
+/// The priority is of type `P`, that must implement `std::cmp::Ord`.
 ///
-/// The item is of type I, that must implement `Hash` and `Eq`.
+/// The item is of type `I`, that must implement `Hash` and `Eq`.
 ///
 /// Implemented as a heap of indexes, stores the items inside an `IndexMap`
 /// to be able to retrieve them quickly.
@@ -169,7 +169,7 @@ where
 
 impl<I, P, H> PriorityQueue<I, P, H> {
     /// Returns an iterator in arbitrary order over the
-    /// (item, priority) elements in the queue
+    /// `(item, priority)` elements in the queue
     pub fn iter(&self) -> Iter<I, P> {
         self.store.iter()
     }
@@ -180,14 +180,14 @@ impl<I, P, H> PriorityQueue<I, P, H> {
         self.store.shrink_to_fit();
     }
 
-    /// Clears the PriorityQueue, returning an iterator over the removed elements in arbitrary order.
+    /// Clears the `PriorityQueue`, returning an iterator over the removed elements in arbitrary order.
     /// If the iterator is dropped before being fully consumed, it drops the remaining elements in arbitrary order.
     pub fn drain(&mut self) -> Drain<I, P> {
         self.store.drain()
     }
 
-    /// Returns the couple (item, priority) with the greatest
-    /// priority in the queue, or None if it is empty.
+    /// Returns the couple `(item, priority)` with the greatest
+    /// priority in the queue, or `None` if it is empty.
     ///
     /// Computes in **O(1)** time
     pub fn peek(&self) -> Option<(&I, &P)> {
@@ -212,7 +212,7 @@ impl<I, P, H> PriorityQueue<I, P, H> {
         self.store.len()
     }
 
-    /// Returns true if the priority queue contains no elements.
+    /// Returns `true` if the priority queue contains no elements.
     pub fn is_empty(&self) -> bool {
         self.store.is_empty()
     }
@@ -277,22 +277,23 @@ where
     P: Ord,
 {
     /// Returns an iterator in arbitrary order over the
-    /// (item, priority) elements in the queue.
+    /// `(item, priority)` elements in the queue.
     ///
     /// The item and the priority are mutable references, but it's a logic error
     /// to modify the item in a way that change the result of `Hash` or `Eq`.
     ///
     /// It's *not* an error, instead, to modify the priorities, because the heap
-    /// will be rebuilt once the `IterMut` goes out of scope. It would be
-    /// rebuilt even if no priority value would have been modified, but the
-    /// procedure will not move anything, but just compare the priorities.
+    /// will be rebuilt once the `IterMut` goes out of scope.
+    ///
+    /// It would be rebuilt even if no priority value would have been modified,
+    /// but the procedure will not move anything, but just compare the priorities.
     pub fn iter_mut(&mut self) -> IterMut<I, P, H> {
         IterMut::new(self)
     }
 
     /// Removes the item with the greatest priority from
-    /// the priority queue and returns the pair (item, priority),
-    /// or None if the queue is empty.
+    /// the priority queue and returns the pair `(item, priority)`,
+    /// or `None` if the queue is empty.
     pub fn pop(&mut self) -> Option<(I, P)> {
         match self.len() {
             0 => None,
@@ -305,7 +306,7 @@ where
         }
     }
 
-    /// Implements a HeapSort.
+    /// Implements a heap sort.
     ///
     /// Returns a `Vec<I>` sorted from the item associated to the highest priority to the lowest.
     pub fn into_sorted_vec(mut self) -> Vec<I> {
@@ -316,7 +317,7 @@ where
         res
     }
 
-    /// Generates a new iterator from self that
+    /// Generates a new iterator from `self` that
     /// will extract the elements from the one with the highest priority
     /// to the lowest one.
     pub fn into_sorted_iter(self) -> IntoSortedIter<I, P, H> {
@@ -330,11 +331,99 @@ where
     I: Hash + Eq,
     H: BuildHasher,
 {
-    /// Insert the item-priority pair into the queue.
+    /// Retains only the elements specified by the `predicate`.
     ///
-    /// If an element equal to `item` was already into the queue,
-    /// it is updated and the old value of its priority returned in `Some`;
-    /// otherwise, returns `None`.
+    /// In other words, remove all elements e for which `predicate(&i, &p)` returns `false`.
+    /// The elements are visited in arbitrary order.
+    pub fn retain<F>(&mut self, predicate: F)
+    where
+        F: FnMut(&I, &P) -> bool,
+    {
+        self.store.retain(predicate);
+        self.heap_build();
+    }
+
+    /// Retains only the elements specified by the `predicate`.
+    ///
+    /// In other words, remove all elements e for which `predicate(&mut i, &mut p)` returns `false`.
+    /// The elements are visited in arbitrary order.
+    ///
+    /// The `predicate` receives mutable references to both the item and
+    /// the priority.
+    ///
+    /// It's a logical error to change the item in a way
+    /// that changes the result of `Hash` or `Eq`.
+    ///
+    /// The `predicate` can change the priority. If the element is retained,
+    /// it will have the updated one.
+    pub fn retain_mut<F>(&mut self, predicate: F)
+    where
+        F: FnMut(&mut I, &mut P) -> bool,
+    {
+        self.store.retain_mut(predicate);
+        self.heap_build();
+    }
+
+    /// Removes the item with the greatest priority from
+    /// the priority queue if the `predicate` returns `true`.
+    ///
+    /// Returns the pair `(item, priority)`, or `None` if the
+    /// queue is empty or the `predicate` returns `false`.
+    ///
+    /// The `predicate` receives mutable references to both the item
+    /// and the priority.
+    ///
+    /// It's a logical error to change the item in a way
+    /// that changes the result of `Hash` or `Eq`.
+    ///
+    /// The `predicate` can change the priority. If it returns `true`,
+    /// the returned couple will have the updated priority,
+    /// otherwise, the heap structural property will be restored.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::PriorityQueue;
+    /// let mut pq = PriorityQueue::new();
+    /// pq.push("Apples", 5);
+    /// pq.push("Bananas", 10);
+    /// assert_eq!(pq.pop_if(|i, p| {
+    ///   *p = 3;
+    ///   false
+    /// }), None);
+    /// assert_eq!(pq.pop(), Some(("Apples", 5)));
+    /// ```
+    pub fn pop_if<F>(&mut self, predicate: F) -> Option<(I, P)>
+    where
+        F: FnOnce(&mut I, &mut P) -> bool,
+    {
+        match self.len() {
+            0 => None,
+            1 => self.store.swap_remove_if(Position(0), predicate),
+            _ => {
+                let r = self.store.swap_remove_if(Position(0), predicate);
+                self.heapify(Position(0));
+                r
+            }
+        }
+    }
+
+    /// Insert the `(item, priority)` pair into the queue.
+    ///
+    /// If an element equal to `item` is already in the queue, its priority
+    /// is updated and the old priority is returned in `Some`; otherwise,
+    /// `item` is inserted with `priority` and `None` is returned.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::PriorityQueue;
+    /// let mut pq = PriorityQueue::new();
+    /// assert_eq!(pq.push("Apples", 5), None);
+    /// assert_eq!(pq.get_priority("Apples"), Some(&5));
+    /// assert_eq!(pq.push("Apples", 6), Some(5));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&6));
+    /// assert_eq!(pq.push("Apples", 4), Some(6));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// ```
     ///
     /// Computes in **O(log(N))** time.
     pub fn push(&mut self, item: I, priority: P) -> Option<P> {
@@ -371,13 +460,29 @@ where
     ///
     /// If an element equal to `item` is already in the queue with a
     /// lower priority, its priority is increased to the new one
-    /// without replacing the element and the old priority is returned.
-    /// Otherwise, the new element is inserted into the queue.
+    /// without replacing the element and the old priority is returned
+    /// in `Some`.
     ///
-    /// Returns `Some` if an element equal to `item` is already in the
-    /// queue. If its priority is higher then `priority`, the latter is returned back,
-    /// otherwise, the old priority is contained in the Option.
-    /// If the item is not in the queue, `None` is returned.
+    /// If an element equal to `item` is already in the queue with an
+    /// equal or higher priority, its priority is not changed and the
+    /// `priority` argument is returned in `Some`.
+    ///
+    /// If no element equal to `item` is already in the queue, the new
+    /// element is inserted and `None` is returned.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::PriorityQueue;
+    /// let mut pq = PriorityQueue::new();
+    /// assert_eq!(pq.push_increase("Apples", 5), None);
+    /// assert_eq!(pq.get_priority("Apples"), Some(&5));
+    /// assert_eq!(pq.push_increase("Apples", 6), Some(5));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&6));
+    /// // Already present with higher priority, so requested (lower)
+    /// // priority is returned.
+    /// assert_eq!(pq.push_increase("Apples", 4), Some(4));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&6));
+    /// ```
     ///
     /// Computes in **O(log(N))** time.
     pub fn push_increase(&mut self, item: I, priority: P) -> Option<P> {
@@ -393,13 +498,29 @@ where
     ///
     /// If an element equal to `item` is already in the queue with a
     /// higher priority, its priority is decreased to the new one
-    /// without replacing the element and the old priority is returned.
-    /// Otherwise, the new element is inserted into the queue.
+    /// without replacing the element and the old priority is returned
+    /// in `Some`.
     ///
-    /// Returns `Some` if an element equal to `item` is already in the
-    /// queue. If its priority is lower then `priority`, the latter is returned back,
-    /// otherwise, the old priority is contained in the Option.
-    /// If the item is not in the queue, `None` is returned.
+    /// If an element equal to `item` is already in the queue with an
+    /// equal or lower priority, its priority is not changed and the
+    /// `priority` argument is returned in `Some`.
+    ///
+    /// If no element equal to `item` is already in the queue, the new
+    /// element is inserted and `None` is returned.
+    ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::PriorityQueue;
+    /// let mut pq = PriorityQueue::new();
+    /// assert_eq!(pq.push_decrease("Apples", 5), None);
+    /// assert_eq!(pq.get_priority("Apples"), Some(&5));
+    /// assert_eq!(pq.push_decrease("Apples", 4), Some(5));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// // Already present with lower priority, so requested (higher)
+    /// // priority is returned.
+    /// assert_eq!(pq.push_decrease("Apples", 6), Some(6));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// ```
     ///
     /// Computes in **O(log(N))** time.
     pub fn push_decrease(&mut self, item: I, priority: P) -> Option<P> {
@@ -410,18 +531,30 @@ where
         }
     }
 
-    /// Change the priority of an Item returning the old value of priority,
+    /// Change the priority of an item returning the old value of priority,
     /// or `None` if the item wasn't in the queue.
     ///
     /// The argument `item` is only used for lookup, and is not used to overwrite the item's data
     /// in the priority queue.
     ///
+    /// # Example
+    /// ```
+    /// # use priority_queue::PriorityQueue;
+    /// let mut pq = PriorityQueue::new();
+    /// assert_eq!(pq.change_priority("Apples", 5), None);
+    /// assert_eq!(pq.get_priority("Apples"), None);
+    /// assert_eq!(pq.push("Apples", 6), None);
+    /// assert_eq!(pq.get_priority("Apples"), Some(&6));
+    /// assert_eq!(pq.change_priority("Apples", 4), Some(6));
+    /// assert_eq!(pq.get_priority("Apples"), Some(&4));
+    /// ```
+    ///
     /// The item is found in **O(1)** thanks to the hash table.
     /// The operation is performed in **O(log(N))** time.
-    pub fn change_priority<Q: ?Sized>(&mut self, item: &Q, new_priority: P) -> Option<P>
+    pub fn change_priority<Q>(&mut self, item: &Q, new_priority: P) -> Option<P>
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
     {
         self.store
             .change_priority(item, new_priority)
@@ -431,18 +564,19 @@ where
             })
     }
 
-    /// Change the priority of an Item using the provided function.
-    /// Return a boolean value where `true` means the item was in the queue and update was successful
+    /// Change the priority of an item using the provided function.
+    ///
+    /// Returns a boolean value where `true` means the item was in the queue and update was successful.
     ///
     /// The argument `item` is only used for lookup, and is not used to overwrite the item's data
     /// in the priority queue.
     ///
     /// The item is found in **O(1)** thanks to the hash table.
     /// The operation is performed in **O(log(N))** time (worst case).
-    pub fn change_priority_by<Q: ?Sized, F>(&mut self, item: &Q, priority_setter: F) -> bool
+    pub fn change_priority_by<Q, F>(&mut self, item: &Q, priority_setter: F) -> bool
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
         F: FnOnce(&mut P),
     {
         self.store
@@ -454,50 +588,52 @@ where
     }
 
     /// Get the priority of an item, or `None`, if the item is not in the queue
-    pub fn get_priority<Q: ?Sized>(&self, item: &Q) -> Option<&P>
+    pub fn get_priority<Q>(&self, item: &Q) -> Option<&P>
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
     {
         self.store.get_priority(item)
     }
 
-    /// Get the couple (item, priority) of an arbitrary element, as reference
+    /// Get the couple `(item, priority)` of an arbitrary element, as reference
     /// or `None` if the item is not in the queue.
-    pub fn get<Q: ?Sized>(&self, item: &Q) -> Option<(&I, &P)>
+    pub fn get<Q>(&self, item: &Q) -> Option<(&I, &P)>
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
     {
         self.store.get(item)
     }
 
-    /// Get the couple (item, priority) of an arbitrary element, or `None`
+    /// Get the couple `(item, priority)` of an arbitrary element, or `None`
     /// if the item was not in the queue.
     ///
     /// The item is a mutable reference, but it's a logic error to modify it
     /// in a way that change the result of  `Hash` or `Eq`.
     ///
     /// The priority cannot be modified with a call to this function.
-    /// To modify the priority use `push`, `change_priority` or
-    /// `change_priority_by`.
-    pub fn get_mut<Q: ?Sized>(&mut self, item: &Q) -> Option<(&mut I, &P)>
+    /// To modify the priority use [`push`](PriorityQueue::push),
+    /// [`change_priority`](PriorityQueue::change_priority) or
+    /// [`change_priority_by`](PriorityQueue::change_priority_by).
+    pub fn get_mut<Q>(&mut self, item: &Q) -> Option<(&mut I, &P)>
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
     {
         self.store.get_mut(item)
     }
 
-    /// Returns the couple (item, priority) with the greatest
-    /// priority in the queue, or None if it is empty.
+    /// Returns the couple `(item, priority)` with the greatest
+    /// priority in the queue, or `None` if it is empty.
     ///
     /// The item is a mutable reference, but it's a logic error to modify it
     /// in a way that change the result of  `Hash` or `Eq`.
     ///
     /// The priority cannot be modified with a call to this function.
-    /// To modify the priority use `push`, `change_priority` or
-    /// `change_priority_by`.
+    /// To modify the priority use[`push`](PriorityQueue::push),
+    /// [`change_priority`](PriorityQueue::change_priority) or
+    /// [`change_priority_by`](PriorityQueue::change_priority_by).
     ///
     /// Computes in **O(1)** time
     pub fn peek_mut(&mut self) -> Option<(&mut I, &P)> {
@@ -512,14 +648,15 @@ where
     }
 
     /// Remove an arbitrary element from the priority queue.
-    /// Returns the (item, priority) couple or None if the item
+    ///
+    /// Returns the `(item, priority)` couple or `None` if the item
     /// is not found in the queue.
     ///
     /// The operation is performed in **O(log(N))** time (worst case).
-    pub fn remove<Q: ?Sized>(&mut self, item: &Q) -> Option<(I, P)>
+    pub fn remove<Q>(&mut self, item: &Q) -> Option<(I, P)>
     where
         I: Borrow<Q>,
-        Q: Eq + Hash,
+        Q: Eq + Hash + ?Sized,
     {
         self.store.remove(item).map(|(item, priority, pos)| {
             if pos.0 < self.len() {
@@ -535,8 +672,8 @@ where
     /// At the end, `other` will be empty.
     ///
     /// **Note** that at the end, the priority of the duplicated elements
-    /// inside self may be the one of the elements in other,
-    /// if other is longer than self
+    /// inside `self` may be the one of the elements in `other`,
+    /// if `other` is longer than `self`
     pub fn append(&mut self, other: &mut Self) {
         self.store.append(&mut other.store);
         self.heap_build();
